/**
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file
 * @author Lukasz Kostyra (l.kostyra@samsung.com)
 * @brief  TeecManager singleton class declaration
 */

#ifndef LIBTEEC_TEEC_MANAGER_H_
#define LIBTEEC_TEEC_MANAGER_H_

#include <unordered_map>
#include <list>
#include <string>
#include <memory>
#include <mutex>
#include <tee_client_api.h>

#include "TeecContext.h"


namespace extension {
namespace libteec {

using TeecContextMap = std::unordered_map<std::string, TeecContextPtr>;
using TeecOperationMap = std::unordered_map<uint32_t, TEEC_Operation>;

class TeecManager final
{
public:
    static TeecManager& Instance();

    TeecContextPtr GetContext(const std::string& contextName);

    uint32_t CreateOperation();
    TEEC_Operation GetOperation(uint32_t id) const;
    void RemoveOperation(uint32_t id);

private:
    TeecManager() = default;
    ~TeecManager() = default;
    TeecManager(const TeecManager&) = delete;
    TeecManager(TeecManager&&) = delete;
    TeecManager& operator=(const TeecManager&) = delete;
    TeecManager& operator=(TeecManager&&) = delete;

    TeecContextMap mContexts;
    TeecOperationMap mOperations;
    mutable std::mutex mOperationListMutex;
};

} // namespace libteec
} // namespace extension

#endif // LIBTEEC_TEEC_MANAGER_H_