/**
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file
 * @author Lukasz Kostyra (l.kostyra@samsung.com)
 * @brief  TeecTempMemoryAllocator class declaration
 */

#ifndef LIBTEEC_TEEC_TEMP_MEMORY_ALLOCATOR_H_
#define LIBTEEC_TEEC_TEMP_MEMORY_ALLOCATOR_H_

#include <vector>
#include <list>
#include <common/picojson.h>


namespace extension {
namespace libteec {

using TempMemoryBuffer = std::vector<uint8_t>;
using TempMemoryList = std::list<TempMemoryBuffer>;

// An object used during OpenSession and InvokeCommand calls
// It's purpose is to temporarily allocate buffers on C++ side
// and keep them in memory until above calls end
class TeecTempMemoryAllocator final
{
public:
    // returns pointer for temp memory
    void* AllocateTempMemory(const picojson::array& array);

private:
    TempMemoryList mMemories;
};

} // namespace libteec
} // namespace extension

#endif // LIBTEEC_TEEC_TEMP_MEMORY_ALLOCATOR_H_